#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.VehicleRouting.Encodings.General;
using HeuristicLab.Problems.VehicleRouting.Interfaces;

namespace HeuristicLab.Problems.VehicleRouting.Encodings.Potvin {
  [Item("PotvinVehicleAssignmentMove", "Item that describes a relocation move on a VRP representation.")]
  [StorableType("C898641C-FE58-4E7C-A8FB-E13412AE3662")]
  public class PotvinVehicleAssignmentMove : Item, IVRPMove {
    [Storable]
    public IVRPEncoding Individual { get; protected set; }

    [Storable]
    public int Tour1 { get; protected set; }

    [Storable]
    public int Tour2 { get; protected set; }


    public PotvinVehicleAssignmentMove()
      : base() {
      Tour1 = -1;
      Tour2 = -1;

      Individual = null;
    }

    public PotvinVehicleAssignmentMove(int tour1, int tour2, PotvinEncoding individual) {
      Tour1 = tour1;
      Tour2 = tour2;

      this.Individual = individual.Clone() as PotvinEncoding;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PotvinVehicleAssignmentMove(this, cloner);
    }

    protected PotvinVehicleAssignmentMove(PotvinVehicleAssignmentMove original, Cloner cloner)
      : base(original, cloner) {
      this.Tour1 = original.Tour1;
      this.Tour2 = original.Tour2;

      this.Individual = cloner.Clone(Individual) as PotvinEncoding;
    }

    [StorableConstructor]
    protected PotvinVehicleAssignmentMove(bool deserializing) : base(deserializing) { }

    #region IVRPMove Members

    public VRPMoveEvaluator GetMoveEvaluator() {
      return new PotvinVehicleAssignmentMoveEvaluator();
    }

    public VRPMoveMaker GetMoveMaker() {
      return new PotvinVehicleAssignmentMoveMaker();
    }

    public ITabuMaker GetTabuMaker() {
      return new PotvinVehicleAssignmentMoveTabuMaker();
    }

    public ITabuChecker GetTabuChecker() {
      return new PotvinVehicleAssignmentMoveTabuCriterion();
    }

    public ITabuChecker GetSoftTabuChecker() {
      PotvinVehicleAssignmentMoveTabuCriterion tabuChecker = new PotvinVehicleAssignmentMoveTabuCriterion();
      tabuChecker.UseAspirationCriterion.Value = true;

      return tabuChecker;
    }

    #endregion
  }
}
